/*
 * Copyright (c) 2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#ifdef HAVE_POCOLIBS
#include <portLib.h>
#endif

#include <velodyne/velodyneLib.h>

#include "velodyne_private.h"

int
velodyne_setup_socket(void)
{
	int r, s;
	struct sockaddr_in addr;

	s = socket(PF_INET, SOCK_DGRAM, 0);
	if (s == -1) {
#ifdef HAVE_POCOLIBS
		errnoSet(errno);
#endif
		return -1;
	}
	addr.sin_family = AF_INET;
	addr.sin_port = htons(VELODYNE_UDP_PORT);
	addr.sin_addr.s_addr = INADDR_ANY;
	memset(addr.sin_zero,'\0',sizeof addr.sin_zero);

	if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) != 0) {
#ifdef HAVE_POCOLIBS
		errnoSet(errno);
#endif
		return -1;
	}
	return s;
}

void
velodyne_close_socket(int fd)
{
	close(fd);
}

int
velodyne_udp_rcv(int s, VELODYNE_PACKET_STR *packet, int flags)
{
	ssize_t nread;
	struct sockaddr_in addr;
	socklen_t addr_size;

	memset(&addr, 0, sizeof(addr));
	addr_size = sizeof addr;

	nread = recvfrom(s, &packet->data, VELODYNE_MSG_BUFFER_SIZE, flags, 
	    (struct sockaddr *)&addr, &addr_size);
	gettimeofday(&packet->ts, NULL);
	printf("read %ld bytes\n", (long)nread);
	return nread;
}
