/*
 * Copyright (c) 2008 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _VELODYNELIB_H
#define _VELODYNELIB_H

#define VELODYNE_NUM_LASERS 32		/* number of lasers per shot */
#define VELODYNE_NUM_SHOTS 12		/* number of shots per packet */
#define VELODYNE_UDP_PORT 2368		/* the UDP port packets are sent to */
#define VELODYNE_MSG_BUFFER_SIZE 1206	/* size of a packet */
/* Magic values coding the laser bank used */
#define VELODYNE_UPPER_HEADER 61183
#define VELODYNE_LOWER_HEADER 56831

#define VELODYNE_FILE 1
#define VELODYNE_NETWORK 2

/**
 * Driver interface descriptor
 */
typedef struct VELODYNE_STR {
	int fd;				/* socket for the network case */
	void *pcap;			/* libpcap descriptor for files */
	int type;			/* FILE or NETWORK */
} VELODYNE_STR;

/**
 * Stuctures describing a data packet 
 */

/* One data point (3 bytes) */
typedef struct VELODYNE_LASER_STR {
	u_int16_t distance;		/* 2mm increments */
	u_int8_t intensity;
} __attribute__((packed)) VELODYNE_LASER_STR;

/* A shot with NUM_LASERS lasers in parallel */
typedef struct VELODYNE_FIRE_STR {
	u_int16_t lower_upper;		/* wich laser bank */
	u_int16_t rotational_pos;	/* 0-35999 */
	struct VELODYNE_LASER_STR lasers[VELODYNE_NUM_LASERS];  /* data */
} __attribute__((packed)) VELODYNE_FIRE_STR;

/* A full packet (payload of a UDP frame) returned by the sensor */
typedef struct VELODYNE_RAW_PACKET_STR {
	struct VELODYNE_FIRE_STR shots[VELODYNE_NUM_SHOTS];
	u_int8_t frac_degrees;
	u_int8_t degrees;
	char status_string[4];
} __attribute__((packed)) VELODYNE_RAW_PACKET_STR;

/* A packet with its timestamp */
typedef struct VELODYNE_PACKET_STR {
	struct timeval ts;
	struct VELODYNE_RAW_PACKET_STR data;
} VELODYNE_PACKET_STR;

/**
 * Higher level data structures
 */

/* Simple cartesian point */
struct VELODYNE_POINT {
	double x, y, z;
};

#ifdef HAVE_POCOLIBS
/**
 * Error codes returned by the library
 */
#define M_velodyneLib 5420

#define S_velodyneLib_NO_MEMORY		((M_velodyneLib << 16) | 1)
#define S_velodyneLib_INVALID_DEVICE	((M_velodyneLib << 16) | 2)
#define S_velodyneLib_FILE_OPEN_ERROR	((M_velodyneLib << 16) | 3)
#define S_velodyneLib_READ_ERROR	((M_velodyneLib << 16) | 4)
#endif

/**
 * Prototypes 
 */
extern VELODYNE_STR *velodyne_open(char *);
extern void velodyne_close(VELODYNE_STR *);
extern void velodyne_print_packet(VELODYNE_PACKET_STR *);
extern void velodyne_print_packet_3d(VELODYNE_PACKET_STR *);
extern void velodyne_compute_coords(u_int16_t, int, int, 
    struct VELODYNE_POINT *);

#endif
